@@ -327,14 +327,26 @@ void Visitor::apply(block128Array& value)
 //
 // Array2Ds
 //
+void Visitor::apply(byteArray2D& value)
+{
+    apply(static_cast<Data&>(value));
+}
 void Visitor::apply(ubyteArray2D& value)
 {
     apply(static_cast<Data&>(value));
 }
+void Visitor::apply(shortArray2D& value)
+{
+    apply(static_cast<Data&>(value));
+}
 void Visitor::apply(ushortArray2D& value)
 {
     apply(static_cast<Data&>(value));
 }
+void Visitor::apply(intArray2D& value)
+{
+    apply(static_cast<Data&>(value));
+}
 void Visitor::apply(uintArray2D& value)
 {
     apply(static_cast<Data&>(value));
@@ -456,14 +468,26 @@ void Visitor::apply(block128Array2D& value)
 //
 // Array3Ds
 //
+void Visitor::apply(byteArray3D& value)
+{
+    apply(static_cast<Data&>(value));
+}
 void Visitor::apply(ubyteArray3D& value)
 {
     apply(static_cast<Data&>(value));
 }
+void Visitor::apply(shortArray3D& value)
+{
+    apply(static_cast<Data&>(value));
+}
 void Visitor::apply(ushortArray3D& value)
 {
     apply(static_cast<Data&>(value));
 }
+void Visitor::apply(intArray3D& value)
+{
+    apply(static_cast<Data&>(value));
+}
 void Visitor::apply(uintArray3D& value)
 {
     apply(static_cast<Data&>(value));
@@ -573,6 +597,10 @@ void Visitor::apply(MatrixTransform& value)
 {
     apply(static_cast<Transform&>(value));
 }
+void Visitor::apply(CoordinateFrame& value)
+{
+    apply(static_cast<Transform&>(value));
+}
 void Visitor::apply(Geometry& value)
 {
     apply(static_cast<Command&>(value));
@@ -679,6 +707,10 @@ void Visitor::apply(TransformSampler& sampler)
 {
     apply(static_cast<AnimationSampler&>(sampler));
 }
+void Visitor::apply(CameraSampler& sampler)
+{
+    apply(static_cast<AnimationSampler&>(sampler));
+}
 void Visitor::apply(Joint& value)
 {
     apply(static_cast<Node&>(value));
@@ -1028,6 +1060,10 @@ void Visitor::apply(LookAt& value)
 {
     apply(static_cast<ViewMatrix&>(value));
 }
+void Visitor::apply(LookDirection& value)
+{
+    apply(static_cast<ViewMatrix&>(value));
+}
 void Visitor::apply(RelativeViewMatrix& value)
 {
     apply(static_cast<ViewMatrix&>(value));
