@@ -10,7 +10,7 @@ THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLI
 
 </editor-fold> */
 
-#include <vsg/io/Options.h>
+#include <vsg/io/FileSystem.h>
 #include <vsg/io/Path.h>
 
 #include <cctype>
